# ##### BEGIN GPL LICENSE BLOCK #####
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
# ##### END GPL LICENSE BLOCK #####

bl_info = {
    "name": "GoB",
    "description": "An unofficial GOZ-like for Blender",
    "author": "ODe",
    "version": (0, 2, 3),
    "blender": (2, 5, 9),
    "api": 38479,
    "location": "At the top info header",
    "warning": "",
    "wiki_url": '',
    "tracker_url": '',
    "category": "Import-Export"}

# ???
# To support reload properly,
# try to access a package var, if it's there, reload everything
if "bpy" in locals():
    import imp
    if "ext_GoB" in locals():
        imp.reload(ext_GoB)
# ???

import bpy,os
from . import ext_GoB

pathGoZ = "C:/Users/Public/Pixologic"
try:
    ficPathZB = open(pathGoZ+"/GoZBrush/GoZ_Config.txt",'rt')
    pathZBrush = '"'+ficPathZB.readline().split('"')[1].replace('\\','/')+'"'
    ficPathZB.close()
except:
    print("Unable to find ZBrush")
    pathZBrush = None
varTime = 0

zbSTarted = False


def applyCavity(me,vcol,magnData):
    basecol = (0.5,0.5,0.5)

    edge_faces = dict([(ed.key, []) for ed in me.edges])
    for face in me.faces:
        magnData.append([0.5]*len(face.vertices))
        for key in face.edge_keys:
            edge_faces[key].append(face.index)
         
    for face in me.faces:
        magn = 0.5
        vertsface = tuple(face.vertices)
        for ed in face.edge_keys:
            if len(edge_faces[ed]) == 2:
                p1 = me.vertices[ed[0]].co
                p2 = me.vertices[ed[1]].co
                c1 = me.faces[edge_faces[ed][0]].center
                c2 = me.faces[edge_faces[ed][1]].center
                c3 = (c1 + c2)/2.
                sens1 = me.vertices[ed[0]].normal.dot(c3 - p1)
                sens2 = me.vertices[ed[1]].normal.dot(c3 - p2)
                magn = -(sens1 + sens2)/2.
                numColor1 = vertsface.index(ed[0])
                numColor2 = vertsface.index(ed[1])
                magnData[face.index][numColor1] = magn
                magnData[face.index][numColor2] = magn
                magn = magn * 50 + 0.5
                if numColor1 == 0:
                    vcol.data[face.index].color1 = magn, magn, magn
                elif numColor1 == 1:
                    vcol.data[face.index].color2 = magn, magn, magn 
                elif numColor1 == 2:
                    vcol.data[face.index].color3 = magn, magn, magn
                elif numColor1 == 3:
                    vcol.data[face.index].color4 = magn, magn, magn
                if numColor2 == 0:
                    vcol.data[face.index].color1 = magn, magn, magn
                elif numColor2 == 1:
                    vcol.data[face.index].color2 = magn, magn, magn
                elif numColor2 == 2:
                    vcol.data[face.index].color3 = magn, magn, magn
                elif numColor2 == 3:
                    vcol.data[face.index].color4 = magn, magn, magn
            else:
                vcol.data[face.index].color1 = basecol
                vcol.data[face.index].color2 = basecol
                vcol.data[face.index].color3 = basecol
                if len(vertsface) == 4:
                    vcol.data[face.index].color4 = basecol

def contrast(intensity,vcol,magnData):
    y = 0.5
    for index,magn in enumerate(magnData):
        vcol.data[index].color1 = magn[0]*intensity+y,magn[0]*intensity+y,magn[0]*intensity+y
        vcol.data[index].color2 = magn[1]*intensity+y,magn[1]*intensity+y,magn[1]*intensity+y
        vcol.data[index].color3 = magn[2]*intensity+y,magn[2]*intensity+y,magn[2]*intensity+y
        if len(magnData[index]) == 4:
            vcol.data[index].color4 = magn[3]*intensity+y,magn[3]*intensity+y,magn[3]*intensity+y

class CavityVertexPaint(bpy.types.Operator):
    bl_idname = "paint.cavity"
    bl_label = "Cavity vertex colors"
    bl_options = {'REGISTER', 'UNDO'}

    intensity = bpy.props.FloatProperty(
            name="Contrast",
            description="Contrast the cavity result",
            min=0.01, max=300.0, default=50.0,)

    def __init__(self):
        self.magnData = []

    def execute(self, context):
        obj = context.object
        if not obj or obj.type != 'MESH':return{'CANCELLED'}
        
        me = obj.data
        if len(me.vertex_colors) > 0:
            vcol = me.vertex_colors[0]
        else:
            vcol = me.vertex_colors.new()
        if not self.magnData:applyCavity(me,vcol,self.magnData)
        else:contrast(self.intensity,vcol,self.magnData)
        return {'FINISHED'}

class GoB_import(bpy.types.Operator):
    bl_idname = "scene.gob_import"
    bl_label = "Import from Zbrush"

    def execute(self,context):
        import os
        global varTime, zbSTarted
        try:
            ficTime = os.path.getmtime(
                    "{0}/GoZBrush/GoZ_ObjectList.txt".format(pathGoZ))
        except:
            print("{0}/GoZBrush/GoZ_ObjectList.txt illisible".format(pathGoZ))
            return{'CANCELLED'}
        print("\n\t------")
        zbSTarted = True
        objectList = []
        if ficTime > varTime:
            fic = open("{0}/GoZBrush/GoZ_ObjectList.txt".format(pathGoZ),'rt')
            line = fic.readline()
            while line:
                objectList.append(line.strip() + '.GoZ')
                line = fic.readline()
            fic.close()
            varTime = ficTime
        else:
            print("Nothing to update")
            return{'CANCELLED'}
        if len(objectList) == 0:
            print("No update")
            return{'CANCELLED'}
        for objPath in objectList:
            ext_GoB.GoZit(objPath)
        return{'FINISHED'}
    def invoke(self, context, event):
        if event.shift:
            # bpy.ops.paint.vertex_color_dirt()
            bpy.ops.paint.cavity()
            return{'FINISHED'}
        else:
            return self.execute(context)

class GoB_export(bpy.types.Operator):
    bl_idname = "scene.gob_export"
    bl_label = "Export to Zbrush"
    def execute(self,context):
        import struct
        global varTime
        scn = bpy.context.scene
        fic = open("{0}/GoZBrush/GoZ_ObjectList.txt".format(pathGoZ),'wt')
        for obj in scn.objects:
            if obj.type == 'MESH' and obj.select:
                obj.name = obj.name.replace('.','_')
                obj.name = obj.name.replace(' ','_')
                bpy.ops.object.mode_set(mode='OBJECT')
                ext_GoB.exportGoZ(
                        pathGoZ,
                        scn,
                        obj,
                        '{0}/GoZProjects/Default'.format(pathGoZ))
                ztn = open(
                    "{0}/GoZProjects/Default/{1}.ztn".format(pathGoZ,obj.name),'wt')
                ztn.write('{0}/GoZProjects/Default/{1}'.format(pathGoZ, obj.name))
                ztn.close()
                fic.write(
                    '{0}/GoZProjects/Default/{1}\n'.format(pathGoZ, obj.name))
        fic.close()
        varTime = os.path.getmtime(
                        "{0}/GoZBrush/GoZ_ObjectList.txt".format(pathGoZ))
        os.system("{0}/GoZBrush/GoZBrushFromApp.exe".format(pathGoZ))
        return{'FINISHED'}
    def invoke(self, context, event):
        if event.shift:
            if pathZBrush is not None:
                os.system('"{0} {1}/GoZApps/Photoshop/GoZ_LoadTextures.zsc"'.format(pathZBrush,pathGoZ))
                return{'FINISHED'}
        else:
            return self.execute(context)
class INFO_HT_header(bpy.types.Header):
    bl_space_type = 'INFO'

    def draw(self, context):
        layout = self.layout

        window = context.window
        scene = context.scene
        rd = scene.render

        row = layout.row(align=True)
        row.template_header()

        if context.area.show_menus:
            sub = row.row(align=True)
            sub.menu("INFO_MT_file")
            sub.menu("INFO_MT_add")
            if rd.use_game_engine:
                sub.menu("INFO_MT_game")
            else:
                sub.menu("INFO_MT_render")
            sub.menu("INFO_MT_help")

        if window.screen.show_fullscreen:
            layout.operator(
                    operator="screen.back_to_previous",
                    icon='SCREEN_BACK',
                    text="Back to Previous")
            layout.separator()
        else:
            layout.template_ID(
                    data=context.window,
                    property="screen",
                    new="screen.new",
                    unlink="screen.delete")
            layout.template_ID(
                    data=context.screen,
                    property="scene",
                    new="scene.new",
                    unlink="scene.delete")

        layout.separator()

        if rd.has_multiple_engines:
            layout.prop(rd, "engine", text="")

        layout.separator()

        layout.template_running_jobs()

        layout.template_reports_banner()

        row = layout.row(align=True)
        row.operator(
                operator="scene.gob_export",
                text="",
                icon='BRUSH_PINCH',
                emboss=False)
        row.operator(
                operator="scene.gob_import",
                text="",
                icon='BLENDER',
                emboss=False)
        row.label(text=scene.statistics())

        # XXX: this should be right-aligned to the RHS of the region
        layout.operator(
                operator="wm.window_fullscreen_toggle",
                icon='FULLSCREEN_ENTER',
                text="")

        # XXX: BEFORE RELEASE, MOVE FILE MENU OUT OF INFO!!!
        """
        sinfo = context.space_data
        row = layout.row(align=True)
        row.prop(sinfo, "show_report_debug", text="Debug")
        row.prop(sinfo, "show_report_info", text="Info")
        row.prop(sinfo, "show_report_operator", text="Operators")
        row.prop(sinfo, "show_report_warning", text="Warnings")
        row.prop(sinfo, "show_report_error", text="Errors")

        row = layout.row()
        row.enabled = sinfo.show_report_operator
        row.operator("info.report_replay")

        row.menu("INFO_MT_report")
        """

def register():
    bpy.utils.register_class(GoB_export)
    bpy.utils.register_class(GoB_import)
    bpy.utils.register_class(INFO_HT_header)
    bpy.utils.register_class(CavityVertexPaint)

def unregister():
    import bl_ui
    bpy.utils.unregister_class(GoB_export)
    bpy.utils.unregister_class(GoB_import)
    bpy.utils.unregister_class(INFO_HT_header)
    bpy.utils.register_class(bl_ui.space_info.INFO_HT_header)
    bpy.utils.unregister_class(CavityVertexPaint)
if __name__ == "__main__":
    register()
